==> sli-plots-0.0.23.tgz: package/addon/components/snap-to-scrubber.js <==
import Component from '@glimmer/component';
import { tracked } from '@glimmer/tracking';
import { bisector } from "d3-array";
import { throttle } from '@ember/runloop';

export default class MultiScrubberComponent extends Component {
  get bisectDate() {
    return bisector(d => typeof d[this.dateKey] === "string" ? new Date(d[this.dateKey]) : d[this.dateKey]).left;
  }

  @tracked fadeState = "opacity-0";
  @tracked currentMousePos;
  @tracked line;
  @tracked circle;
  @tracked positions = { x: 0, y: 0 }; // position x/y of line and circle

  // ~~~~~~~~~~~~~~~~~~~~
  // CONFIGURATIONS
  // ~~~~~~~~~~~~~~~~~~~~
  get strokeWidth() {
    return this.args.width || '1';
  }

  get lineColor() {
    return this.args.lineColor || 'gray';
  }

  get valKey() {
    return this.args.valKey || 'value';
  }

  get dateKey() {
    return this.args.dateKey || 'at';
  }

  get circleRadius() {
    return this.args.circleRadius || 2.5;
  }

  get circleColor() {
    return this.args.circleColor || 'gray';
  }

  get fullHeightLine() {
    return this.args.fullHeightLine || false;
  }

  get hideLine() {
    return this.args.hideLine;
  }

  // ~~~~~~~~~~~~~~~~~~~~
  // COMPUTED
  // ~~~~~~~~~~~~~~~~~~~~
  get primaryDataset() {
      return this.args.datasets ? this.args.datasets[0] : [];
  }
  
  get svg() {
    return this.args.svg
      ? this.args.svg
      : this.line ? this.findSvg() : null;
  }

  get svgDimensions() {
    return this.svg ? this.svg.getBoundingClientRect() : {};  
  }

  get chartWidth() {
    return this.args.chartWidth || this.svgDimensions.width;
  }

  get isMultiScale() {
    return typeof this.args.xScale === 'object';
  }

  get chartBottom() {
    return this.svgDimensions.height;
  }

  // ~~~~~~~~~~~~~~~~~~~~
  // INTERACTION EVENTS
  // ~~~~~~~~~~~~~~~~~~~~

  mousemove = (e) => {
    throttle(this, this.updateLine, e, 20);
  }

  mouseout = () => {
    this.fadeState = "opacity-0";
  }

  mouseover = (e) => {
    if(e.type === "touchstart") {
      this.currentMousePos = this.mouseOrTouchPosition(e);
      this.setMovingElementsPosition(this.currentMousePos);
    }
    this.fadeState = "opacity-100";
  }

  initElement = (element, [elementName]) => {
    this[elementName] = element;
  }

  // Move line and update text values
  updateLine = (e) => {
    const mousePos = this.mouseOrTouchPosition(e);
    this.currentMousePos = mousePos;
    if( mousePos > 0.5 ) {
      const nearestPoints = this.findNearestDatapoints(mousePos);
      this.setElementPositions(nearestPoints)
      // updateCurrentValues - function that returns selected datapoints
      if(this.args.updateCurrentValues) this.args.updateCurrentValues(nearestPoints)
    }
  }

  setElementPositions(nearestPoints) {
    const dataPoint = nearestPoints[0];
    this.positions = {
      x: dataPoint ? dataPoint.x : -100,
      y: dataPoint ? dataPoint.y : -100
    }
  }

  // For each dataset, return nearest dataset points to scrubber, alongside x position, y position, color, and text value
  findNearestDatapoints(mousePos) {
    if (!this.args.datasets) return [{x: "", y: ""}];

    return this.args.datasets.map((dataset, index) => {
      const xScale = this.isMultiScale ? this.args.xScale[index] : this.args.xScale;
      const x0 = xScale.invert(mousePos); // x position within the element.
      const dataPoint = this.nearestDataPoint(dataset, x0);
      if (!dataPoint) return
      const time = typeof dataPoint[this.dateKey] === "string" ? new Date(dataPoint[this.dateKey]) : dataPoint[this.dateKey]
      const x = xScale(time);
      return {
        ...dataPoint,
        x: x < 0 ? 0 : x,
        y: this.args.yScale(dataPoint[this.valKey])
      }
    }).filter((datapoint) => datapoint);
  }

  // Get nearest dataset point to date x0
  nearestDataPoint(dataset, x0) {
    const i = this.bisectDate(dataset, x0, 1);
    const d0 = dataset[i - 1];
    const d1 = dataset[i];
    // out of dataset bounds
    if (dataset.length === 0 || x0 < dataset[0][this.dateKey] || x0 > dataset[dataset.length - 1][this.dateKey]) return
    return (d1 && d0) ? (x0 - new Date(d0[this.dateKey]) > new Date(d1[this.dateKey]) - x0 ? d1 : d0) : 0;
  }

  // Returns a mouse or touch position relative to the svg
  mouseOrTouchPosition(event) {
    const clientX = event.type.includes("touch") ? event.targetTouches[0].clientX : event.clientX;
    return clientX - this.svg.getBoundingClientRect().left;
  }

  // Get parent svg element based on inited element
  findSvg() {
    return this.line.closest('svg')
  }
}

==> sli-plots-0.0.26.tgz: package/addon/components/snap-to-scrubber.js <==
import Component from '@glimmer/component';
import { tracked } from '@glimmer/tracking';
import { bisector } from "d3-array";
import { throttle } from '@ember/runloop';

export default class MultiScrubberComponent extends Component {
  get bisectDate() {
    return bisector(d => typeof d[this.dateKey] === "string" ? new Date(d[this.dateKey]) : d[this.dateKey]).left;
  }

  @tracked fadeState = "opacity-0";
  @tracked currentMousePos;
  @tracked line;
  @tracked circle;
  @tracked positions = { x: 0, y: 0 }; // position x/y of line and circle

  // ~~~~~~~~~~~~~~~~~~~~
  // CONFIGURATIONS
  // ~~~~~~~~~~~~~~~~~~~~
  get strokeWidth() {
    return this.args.width || '1';
  }

  get lineColor() {
    return this.args.lineColor || 'gray';
  }

  get valKey() {
    return this.args.valKey || 'SUM';
  }

  get dateKey() {
    return this.args.dateKey || 'OCCURED';
  }

  get circleRadius() {
    return this.args.circleRadius || 2.5;
  }

  get circleColor() {
    return this.args.circleColor || 'gray';
  }

  get fullHeightLine() {
    return this.args.fullHeightLine || false;
  }

  get hideLine() {
    return this.args.hideLine;
  }

  // ~~~~~~~~~~~~~~~~~~~~
  // COMPUTED
  // ~~~~~~~~~~~~~~~~~~~~
  get primaryDataset() {
      return this.args.datasets ? this.args.datasets[0] : [];
  }
  
  get svg() {
    return this.args.svg
      ? this.args.svg
      : this.line ? this.findSvg() : null;
  }

  get svgDimensions() {
    return this.svg ? this.svg.getBoundingClientRect() : {};  
  }

  get chartWidth() {
    return this.args.chartWidth || this.svgDimensions.width;
  }

  get isMultiScale() {
    return typeof this.args.xScale === 'object';
  }

  get chartBottom() {
    return this.svgDimensions.height;
  }

  // ~~~~~~~~~~~~~~~~~~~~
  // INTERACTION EVENTS
  // ~~~~~~~~~~~~~~~~~~~~

  mousemove = (e) => {
    throttle(this, this.updateLine, e, 20);
  }

  mouseout = () => {
    this.fadeState = "opacity-0";
  }

  mouseover = (e) => {
    if(e.type === "touchstart") {
      this.currentMousePos = this.mouseOrTouchPosition(e);
      this.setMovingElementsPosition(this.currentMousePos);
    }
    this.fadeState = "opacity-100";
  }

  initElement = (element, [elementName]) => {
    this[elementName] = element;
  }

  // Move line and update text values
  updateLine = (e) => {
    const mousePos = this.mouseOrTouchPosition(e);
    this.currentMousePos = mousePos;
    if( mousePos > 0.5 ) {
      const nearestPoints = this.findNearestDatapoints(mousePos);
      this.setElementPositions(nearestPoints)
      // updateCurrentValues - function that returns selected datapoints
      if(this.args.updateCurrentValues) this.args.updateCurrentValues(nearestPoints)
    }
  }

  setElementPositions(nearestPoints) {
    const dataPoint = nearestPoints[0];
    this.positions = {
      x: dataPoint ? dataPoint.x : -100,
      y: dataPoint ? dataPoint.y : -100
    }
  }

  // For each dataset, return nearest dataset points to scrubber, alongside x position, y position, color, and text value
  findNearestDatapoints(mousePos) {
    if (!this.args.datasets) return [{x: "", y: ""}];

    return this.args.datasets.map((dataset, index) => {
      const xScale = this.isMultiScale ? this.args.xScale[index] : this.args.xScale;
      const x0 = xScale.invert(mousePos); // x position within the element.
      const dataPoint = this.nearestDataPoint(dataset, x0);
      if (!dataPoint) return
      const time = typeof dataPoint[this.dateKey] === "string" ? new Date(dataPoint[this.dateKey]) : dataPoint[this.dateKey]
      const x = xScale(time);
      return {
        ...dataPoint,
        x: x < 0 ? 0 : x,
        y: this.args.yScale(dataPoint[this.valKey])
      }
    }).filter((datapoint) => datapoint);
  }

  // Get nearest dataset point to date x0
  nearestDataPoint(dataset, x0) {
    const i = this.bisectDate(dataset, x0, 1);
    const d0 = dataset[i - 1];
    const d1 = dataset[i];
    // out of dataset bounds
    if (dataset.length === 0 || x0 < dataset[0][this.dateKey] || x0 > dataset[dataset.length - 1][this.dateKey]) return
    return (d1 && d0) ? (x0 - new Date(d0[this.dateKey]) > new Date(d1[this.dateKey]) - x0 ? d1 : d0) : 0;
  }

  // Returns a mouse or touch position relative to the svg
  mouseOrTouchPosition(event) {
    const clientX = event.type.includes("touch") ? event.targetTouches[0].clientX : event.clientX;
    return clientX - this.svg.getBoundingClientRect().left;
  }

  // Get parent svg element based on inited element
  findSvg() {
    return this.line.closest('svg')
  }
}
